/********************************************************************/
/*                                                                  */
/*      Timer Header for KS32C5000                                  */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*      Modified, programmed by Young-sun Kim                       */
/*                                                                  */
/*      Description : 1990-02-03 first edited                       */ 
/********************************************************************/

#ifndef _TIMER_H_
#define _TIMER_H_

#include "ks5000.h"

/* Timer channel number */
#define  TIMER0       0
#define  TIMER1       1

/* Timer Mode Register */

#define  TM0_RUN      0x01  /* Timer 0 enable */
#define  TM0_TOGGLE   0x02  /* 0, interval mode */
#define  TM0_OUT_1    0x04  /* Timer 0 Initial TOUT0 value */
#define  TM1_RUN      0x08  /* Timer 1 enable */
#define  TM1_TOGGLE   0x10  /* 0, interval mode */
#define  TM1_OUT_1    0x20  /* Timer 0 Initial TOUT0 value */

/*===================================*/
/*	Usable macros functions      */
/*===================================*/

#define Timer0Stop()        (TMOD &= ~TM0_RUN)
#define Timer1Stop()        (TMOD &= ~TM1_RUN)

#define Timer0Start()       (TMOD |= TM0_RUN)
#define Timer1Start()       (TMOD |= TM1_RUN)

#define TimerStart(t)       ((t)? Timer1Start():Timer0Start())
#define TimerStop(t)        ((t)? Timer1Stop(): Timer0Stop())


/* Timer frequency value */
#define tMCLK         (1.0/fMCLK)
#define tmScale_us(t) (t*1000000)
#define tmScale_ms(t) (t*1000)

#define TM0_FREQ     (fMCLK/TDATA0) 
#define TM1_FREQ     (fMCLK/TDATA1)

#define Freq_IntervalMode(td)  (fMCLK/(2*td)) 
#define Freq_ToggleMode(td)    (fMCLK/td) 
#define tmDATA(time)           ((time/tMCLK)-1) 

typedef struct {
      U32 tmdata;     // Timer data,timer range is
                      // tmdata[1~0xffffffff]
      U32 tmtoggle;   // default interval mode
      U32 toutvalue;  // Timer 0 TOUT0 value
      U32 toen;       // Timer out enable,changed port mode & iopcon   
      U32 tmintmask;  // enable timer interrupt mask  
      U32 tmintmode;  // if set, fast interrupt mode
}TM_PARAM; 


typedef struct {
      unsigned int tick;
      unsigned int min;
      unsigned int hour;
      unsigned int day;
      unsigned int month;

}TIME; 



/*===================================*/
/*Timer function prototype functions */
/*===================================*/
extern void Rtc_init(int /* timer device number */);
extern void tm_init_ms(int /* timer device number */);
/* 
 * Initialize timer function with default value as follows:
 * Clear Timer counter(TCNT),interrupt mode and mask bit set.
 * Timer mode(TMOD) is toggle mode, initial timer output(TOUT) state,
 * and timer run enable.
 * Timer data register(TDATA) is set 0xfbc51f for 1sec timer interval.
 */

extern void ClrTimeVar(void);
extern void tmReset(int /* timer device number */);
/* 
 * Timer reset function. it clear timer counter and interrupt pending register
 * and restart timer 
 */


extern void tmCntr(char /* timer  number */,TM_PARAM * /* timer parameter */);
/* 
 * Timer control function. It will be set the timer mode, time periode 
 * ,timer interrupt mode and mask. And also set toen to output timer count
 *  value to port 
 */

extern float tmPeriod(U32 /* timer data register value */);
/*
 * Return timer period for given timer data register value. 
 */


extern void TimerStartUpDialog(char /*timer device number*/);
/* Start up dialog function for set timer control value */


/*===================================*/
/*  Timer interrupt service routines */
/*===================================*/

void  tm0isr(void);    /* timer0 interrupt service */
void  tm1isr(void);    /* timer1 interrupt service */

extern void tmConfig(void);
/* Timer configuration monitoring routines */

extern void TimerTest(void);
/* Timer test top routine */


extern void DisplayTime(char /*timer*/);
/* display current time to console */

extern char *getTimeSec(char /*timer*/);
/* get current time [sec] */

extern char *getTimeMs(char /*timer*/);
/* get current time [ms] */


/* Timer for Dhryston Program : Time unit[sec]*/
extern unsigned long TimeStopCheck(int TMn) ;


#endif  /* _TIMER_H_ */

